"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Handler = void 0;
const typescript_aws_apigateway_helper_1 = require("typescript-aws-apigateway-helper");
const typescript_aws_cloudwatch_helper_1 = require("typescript-aws-cloudwatch-helper");
const typescript_aws_dynamo_helper_1 = require("typescript-aws-dynamo-helper");
const typescript_aws_kms_helper_1 = require("typescript-aws-kms-helper");
const typescript_aws_lambda_helper_1 = require("typescript-aws-lambda-helper");
const typescript_aws_s3_helper_1 = require("typescript-aws-s3-helper");
const typescript_aws_ses_helper_1 = require("typescript-aws-ses-helper");
const typescript_aws_sns_helper_1 = require("typescript-aws-sns-helper");
const typescript_aws_sqs_helper_1 = require("typescript-aws-sqs-helper");
const typescript_aws_ssm_helper_1 = require("typescript-aws-ssm-helper");
const typescript_aws_sts_helper_1 = require("typescript-aws-sts-helper");
const typescript_ilogger_1 = require("typescript-ilogger");
const typescript_helper_functions_1 = require("typescript-helper-functions");
/**
 * Handles an incoming lambda execution request
 */
class Handler {
    /**
     * Initialize a new instance of Handler
     * @param LogLevel {LogLevel} Log Level
     */
    // tslint:disable-next-line: no-shadowed-variable
    constructor(LogLevel) {
        this.LogLevel = LogLevel;
        // initialize the Logger
        this.Logger = new typescript_ilogger_1.Logger(this.LogLevel);
        // initialize all the helpers
        this.APIGatewayHelper = new typescript_aws_apigateway_helper_1.APIGatewayHelper(this.Logger);
        this.CloudWatchHelper = new typescript_aws_cloudwatch_helper_1.CloudWatchHelper(this.Logger);
        this.DynamoHelper = new typescript_aws_dynamo_helper_1.DynamoHelper(this.Logger);
        this.KMSHelper = new typescript_aws_kms_helper_1.KMSHelper(this.Logger);
        this.LambdaHelper = new typescript_aws_lambda_helper_1.LambdaHelper(this.Logger);
        this.S3Helper = new typescript_aws_s3_helper_1.S3Helper(this.Logger);
        this.SESHelper = new typescript_aws_ses_helper_1.SESHelper(this.Logger);
        this.SNSHelper = new typescript_aws_sns_helper_1.SNSHelper(this.Logger);
        this.SQSHelper = new typescript_aws_sqs_helper_1.SQSHelper(this.Logger);
        this.SSMHelper = new typescript_aws_ssm_helper_1.SSMHelper(this.Logger);
        this.STSHelper = new typescript_aws_sts_helper_1.STSHelper(this.Logger);
        this.LogHelper = new typescript_helper_functions_1.LogHelper(this.Logger);
        this.ObjectOperations = new typescript_helper_functions_1.ObjectOperations();
    }
    /**
     * Execute a lambda
     * @param payload {T} Payload to use
     * @param context {Context} AWS Context
     * @param callback {Callback} AWS Callback
     * @param actionToExecute {function} Action to take
     */
    Execute(payload, context, callback, actionToExecute) {
        return __awaiter(this, void 0, void 0, function* () {
            const action = `${Handler.name}.${this.Execute.name}`;
            this.LogHelper.LogInputs(action, { payload, context, callback });
            if (this.ObjectOperations.IsNullOrEmpty(context)) {
                throw new Error(`[${action}]-Must supply context`);
            }
            let result;
            let error;
            try {
                result = yield actionToExecute();
                this.Logger.Trace(`[${action}]-${JSON.stringify(result)}`);
            }
            catch (err) {
                error = err;
                this.Logger.Error(`[${action}]-${JSON.stringify(error)}`);
            }
            finally {
                callback(error, result);
            }
        });
    }
    /**
     * Orchestrate a function
     * @param actionToExecute {function} Action to take
     */
    Orchestrate(actionToExecute) {
        return __awaiter(this, void 0, void 0, function* () {
            const action = `${Handler.name}.${this.Orchestrate.name}`;
            let result;
            try {
                this.Logger.Information(`[${action}]-Starting orchestration...`);
                result = yield actionToExecute();
            }
            finally {
                this.Logger.Information(`[${action}]-Finished orchestration...`);
            }
            return result;
        });
    }
    /**
     * Orchestrate function to process records off SQS queue
     * @param records {SQSRecord[]} Records to process
     * @param queueUrl {string} Queue URL of record set
     * @param actionToExecute {function} Action to perform on each record
     */
    OrchestrateSQS(records, queueUrl, actionToExecute) {
        return __awaiter(this, void 0, void 0, function* () {
            const action = `${Handler.name}.${this.OrchestrateSQS.name}`;
            if (this.ObjectOperations.IsNullOrWhitespace(queueUrl)) {
                throw new Error(`[${action}]-Must supply queueUrl`);
            }
            yield this.Orchestrate(() => __awaiter(this, void 0, void 0, function* () {
                for (const record of records) {
                    this.Logger.Trace(`[${action}]-Record Body: ${record.body}`);
                    // run the code
                    yield actionToExecute();
                    // delete the message
                    yield this.SQSHelper.DeleteMessageAsync(queueUrl, record.receiptHandle);
                }
            }));
        });
    }
}
exports.Handler = Handler;
//# sourceMappingURL=handler.js.map